using System;
using System.Configuration;
using System.Data;
using System.Data.Common;

namespace Marwin.Routines.Data {
	public class Data : IDisposable {

		private string _connectionStringName { get; set; }
		private DbProviderFactory _factory { get; set; }
		private DbConnection _connection { get; set; }

		public Data(string connectionStringName) {
			if (string.IsNullOrEmpty(connectionStringName)) throw new ArgumentNullException();
			_connectionStringName = connectionStringName;
			_factory = GetFactory(connectionStringName);
			_connection = CreateDbConnection(connectionStringName, _factory);
		}

		public void Dispose() {
			_connection.Dispose();
		}

		public DataTable GetTable(string query, DbParameter[] parameters) {
		   return GetTable(
				_connection,
				_factory,
				query,
				parameters
			);
		}

		public Object GetSingleValue(string query, DbParameter[] parameters) {
		   return GetSingleValue(
				_connection,
				_factory,
				query,
				parameters
			);
		}

		public void Execute(string query, DbParameter[] parameters) {
		   Execute(
				_connection,
				_factory,
				query,
				parameters
			);
		}
		public DbParameter CreateParameter(string name, DbType type, object value) {
			DbParameter parameter = _factory.CreateParameter();
			parameter.DbType = type;
			parameter.ParameterName = name;
			parameter.Value = value;
			return parameter;
		}
		public DbParameter CreateParameter(string name, DbType type, int size, object value) {
			DbParameter parameter = _factory.CreateParameter();
			parameter.DbType = type;
			parameter.ParameterName = name;
			parameter.Size = size;
			parameter.Value = value;
			return parameter;
		}
		private string GetConnectionString(string connectionStringName) {
			string conn = string.Empty;
			if (string.IsNullOrEmpty(connectionStringName)) throw new ArgumentNullException();
			if (ConfigurationManager.ConnectionStrings[connectionStringName] == null) throw new SystemException(string.Format("Connection string with name {0} was not found in config file.", connectionStringName));
			return ConfigurationManager.ConnectionStrings[connectionStringName].ConnectionString;
		}
		private DbProviderFactory GetFactory(string connectionStringName) {
			if (string.IsNullOrEmpty(connectionStringName)) throw new ArgumentNullException();
			if (ConfigurationManager.ConnectionStrings[connectionStringName] == null) throw new SystemException(string.Format("Connection string with name {0} was not found in config file.", connectionStringName));
			if (string.IsNullOrEmpty(ConfigurationManager.ConnectionStrings[connectionStringName].ProviderName)) throw new SystemException(string.Format("Connection string with name {0} has no provider defined.", connectionStringName));
			return DbProviderFactories.GetFactory(ConfigurationManager.ConnectionStrings[connectionStringName].ProviderName);
		}
		private DbConnection CreateDbConnection(string connectionStringName) {

			// Create the DbProviderFactory and DbConnection.
			DbProviderFactory factory = GetFactory(connectionStringName);

			DbConnection connection = factory.CreateConnection();
			connection.ConnectionString = GetConnectionString(connectionStringName);
			// Return the connection.
			return connection;
		}

		private DbConnection CreateDbConnection(string connectionStringName, DbProviderFactory factory) {
			DbConnection connection = factory.CreateConnection();
			connection.ConnectionString = GetConnectionString(connectionStringName);
			// Return the connection.
			return connection;
		}

		private Object GetSingleValue(DbConnection connection, DbProviderFactory factory, string query, DbParameter[] parameters) {
			if (connection.State != ConnectionState.Open) connection.Open();

			DbCommand command = factory.CreateCommand();
			command.Connection = connection;
			command.CommandText = query;
			
			SetPrameters(parameters, command);
			object returnValue = command.ExecuteScalar();
			
			connection.Close();
			connection.Dispose();

			return returnValue;
		}

		private DataTable GetTable(DbConnection connection, DbProviderFactory factory, string query, DbParameter[] parameters) {
			if (connection.State != ConnectionState.Open) connection.Open();

			DbCommand command = factory.CreateCommand();
			command.Connection = connection;
			command.CommandText = query;
			SetPrameters(parameters, command);
			DbDataAdapter adapter = factory.CreateDataAdapter();
			adapter.SELECTCommand = command;

			DataTable table = new DataTable();
			adapter.Fill(table);

			connection.Close();
			connection.Dispose();

			return table;
		}

		private void Execute(DbConnection connection, DbProviderFactory factory, string query, DbParameter[] parameters) {
			if (connection.State != ConnectionState.Open) connection.Open();

			DbCommand command = factory.CreateCommand();
			command.Connection = connection;
			command.CommandText = query;
			SetPrameters(parameters, command);
			DbDataAdapter adapter = factory.CreateDataAdapter();
			adapter.SELECTCommand = command;

			command.ExecuteNonQuery();

			connection.Close();
			connection.Dispose();
		}
		private static void SetPrameters(DbParameter[] parameters, DbCommand command) {
			if (parameters != null) {
				foreach (DbParameter parameter in parameters) {
					// null values
					if (parameter.Value == null) parameter.Value = DBNull.Value;
					command.Parameters.Add(parameter);
				}
			}
		}
	}
	[Serializable]
	public class NoDataFoundException : System.Data.DataException {
		public NoDataFoundException() { }
		public NoDataFoundException(string message) : base(message) { }
		public NoDataFoundException(string message, Exception INNER) : base(message, INNER) { }
		protected NoDataFoundException(
			System.Runtime.Serialization.SerializationInfo info,
			System.Runtime.Serialization.StreamingContext context)
			: base(info, context) { }
	}
}

